<?php

class ArrayPieceSquadro implements ArrayAccess, Countable {
    private $pieces = [];

    // Méthode pour vérifier si un offset existe (compatible avec ArrayAccess)
    public function offsetExists(mixed $offset): bool {
        return isset($this->pieces[$offset]);
    }

    // Méthode pour récupérer une pièce à un offset donné (compatible avec ArrayAccess)
    public function offsetGet(mixed $offset): mixed {
        return $this->pieces[$offset] ?? null;
    }

    // Méthode pour définir une pièce à un offset donné (compatible avec ArrayAccess)
    public function offsetSet(mixed $offset, mixed $value): void {
        if ($value instanceof PieceSquadro) {
            if (is_null($offset)) {
                $this->pieces[] = $value; // Ajoute à la fin du tableau si l'offset est null
            } else {
                $this->pieces[$offset] = $value; // Définit la valeur à l'offset spécifié
            }
        } else {
            throw new InvalidArgumentException("Value must be an instance of PieceSquadro.");
        }
    }

    // Méthode pour supprimer une pièce à un offset donné (compatible avec ArrayAccess)
    public function offsetUnset(mixed $offset): void {
        unset($this->pieces[$offset]);
    }

    // Méthode pour compter le nombre de pièces (compatible avec Countable)
    public function count(): int {
        return count($this->pieces);
    }

    // Méthode pour ajouter une pièce au tableau
    public function add(PieceSquadro $piece): void {
        $this->pieces[] = $piece;
    }

    // Méthode pour supprimer une pièce à un index donné
    public function remove(int $index): void {
        if (isset($this->pieces[$index])) {
            unset($this->pieces[$index]);
            $this->pieces = array_values($this->pieces); // Réindexe le tableau après suppression
        }
    }

    // Méthode pour convertir le tableau de pièces en JSON
    public function toJson(): string {
        return json_encode(array_map(function($piece) {
            return $piece->toJson();
        }, $this->pieces));
    }

    // Méthode pour créer une instance de ArrayPieceSquadro à partir d'un JSON
    public static function fromJson(string $json): self {
        $data = json_decode($json, true);
        $array = new self();
        foreach ($data as $pieceJson) {
            $array->add(PieceSquadro::fromJson($pieceJson));
        }
        return $array;
    }
}
?>